using System;
using System.Drawing;
using System.Windows.Forms;

public class AppForm : Form
{
	// Members
	private GameOfLife gol;
	private GOLView golView;
	private GOLController golController;
	
    // C'tor
    public AppForm()
    {
		this.Text = "Game Of Life v1.0";
		this.Size = new Size(800, 600);

		// Add Controls
		
		// Model
		this.gol = new GameOfLife(new Size(80, 80));
		
		// View
		this.golView = new GOLView(this.gol);
		this.golView.Dock = DockStyle.Fill;
		this.golView.BackColor = Color.White;
		this.Controls.Add(this.golView);
		
		// Controller
		this.golController = new GOLController(this.gol);
		this.golController.Dock = DockStyle.Right;
		this.golController.BackColor = Color.LightGray;
		this.Controls.Add(this.golController);
		
		// Connect Events
		this.golController.Clear += new CommandHandler(this.gol.Clear);
		this.golController.Next += new CommandHandler(this.gol.BuildNextGeneration);
		this.golController.StartAutoMode += new StartAutoModeCommandHandler(this.gol.AutoModeStart);
		this.golController.StopAutoMode += new CommandHandler(this.gol.AutoModeStop);
		this.golController.SetStartPattern += new StartPatternHandler(this.gol.SetStartPattern);
		
		this.gol.NextGenerationCompleted += new NextGenerationCompletedHandler(this.golView.Invalidate);
		this.gol.NextGenerationCompleted += new NextGenerationCompletedHandler(this.golController.InvokeNrOfGeneration);
    }
}