using System;
using System.Windows.Forms;
using System.Drawing;

public delegate void CommandHandler();
public delegate void StartPatternHandler(StartPatterns sp);
public delegate void StartAutoModeCommandHandler(int msPerGeneration);
public delegate void SetNrOfGenerationHandler(int nrOfGeneration);

class GOLController : Panel
{
	// Members
	private GameOfLife gol;
	private Button load;
	private Button save;
	private Button next;
	private Button clear;
	private ComboBox startPattern;
	private StartPatterns[] availablePatterns;
	private Button autoMode;
	private TextBox msPerGeneration;
	private bool doingAutoMode;
	private Label nrOfGeneration;
	
	// Events
	public event CommandHandler Clear;
	public event CommandHandler Next;
	public event StartPatternHandler SetStartPattern;
	public event StartAutoModeCommandHandler StartAutoMode;
	public event CommandHandler StopAutoMode;
	
	// C'tor
	public GOLController(GameOfLife gol)
	{
		this.gol = gol;
		
		this.doingAutoMode = false;
		
		int margin = 5;
		int width = 100;

		this.load = new Button();
		this.load.Text = "Load Pattern";
		this.load.Location = new Point(margin, margin);
		this.load.Width = width;
		this.load.Click += new EventHandler(load_Click);
		this.Controls.Add(this.load);

		this.save = new Button();
		this.save.Text = "Save Pattern";
		this.save.Location = new Point(margin, this.load.Bottom + margin);
		this.save.Width = width;
		this.save.Click += new EventHandler(save_Click);
		this.Controls.Add(this.save);
		
		this.clear = new Button();
		this.clear.Text = "Clear";
		this.clear.Location = new Point(margin, this.save.Bottom + margin);
		this.clear.Width = width;
		this.clear.Click += new EventHandler(clear_Click);
		this.Controls.Add(this.clear);
		
		this.next = new Button();
		this.next.Text = "Next";
		this.next.Location = new Point(margin, this.clear.Bottom + margin);
		this.next.Width = width;
		this.next.Click += new EventHandler(next_Click);
		this.Controls.Add(this.next);
		
		this.startPattern = new ComboBox();
		this.startPattern.DropDownStyle = ComboBoxStyle.DropDownList;
		this.startPattern.Location = new Point(margin, this.next.Bottom + margin);
		this.startPattern.Width = width;
		this.startPattern.SelectedIndexChanged += new EventHandler(startPattern_SelectedIndexChanged);

		string[] patternNames = Enum.GetNames(typeof(StartPatterns));
		this.availablePatterns = new StartPatterns[patternNames.Length];
		for (int i=0; i<patternNames.Length; i++)
		{
			this.availablePatterns[i] = (StartPatterns) Enum.Parse(typeof(StartPatterns), patternNames[i]);
			this.startPattern.Items.Add(patternNames[i]);
		}

		this.Controls.Add(this.startPattern);
		
		this.autoMode = new Button();
		this.autoMode.Text = "Start Automode";
		this.autoMode.Location = new Point(margin, this.startPattern.Bottom + margin);
		this.autoMode.Width = width;
		this.autoMode.Click += new EventHandler(autoMode_Click);
		this.Controls.Add(this.autoMode);
		
		this.msPerGeneration = new TextBox();
		this.msPerGeneration.TextAlign = HorizontalAlignment.Right;
		this.msPerGeneration.Text = "1000";
		this.msPerGeneration.Location = new Point(margin, this.autoMode.Bottom + margin);
		this.msPerGeneration.Width = width - 25;
		this.Controls.Add(this.msPerGeneration);
		
		Label l = new Label();
		l.Text = "ms";
		l.Location = new Point(this.msPerGeneration.Right + margin, this.msPerGeneration.Top);
		l.Width = 20;
		l.Height = this.msPerGeneration.Height;
		// l.BackColor = Color.Red;		// debug
		this.Controls.Add(l);

		this.nrOfGeneration = new Label();
		this.nrOfGeneration.Text = "Generation: #0";
		this.nrOfGeneration.Location = new Point(margin, l.Bottom + margin);
		this.nrOfGeneration.Width = width;
		this.Controls.Add(this.nrOfGeneration);
	}

	void load_Click(object sender, EventArgs e)
	{
		OpenFileDialog ofd = new OpenFileDialog();
		ofd.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
		
		if (ofd.ShowDialog() != DialogResult.OK)
			return;
		
		this.gol.LoadPattern(ofd.FileName);
	}

	void save_Click(object sender, EventArgs e)
	{
		SaveFileDialog sfd = new SaveFileDialog();
		sfd.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";

		if (sfd.ShowDialog() != DialogResult.OK)
			return;

		this.gol.SavePattern(sfd.FileName);
	}
	
	void clear_Click(object sender, EventArgs e)
	{
		if (this.Clear != null)
			this.Clear();
	}
	
	void next_Click(object sender, EventArgs e)
	{
		if (this.Next != null)
			this.Next();
	}

	void startPattern_SelectedIndexChanged(object sender, EventArgs e)
	{
		if (this.SetStartPattern != null)
			this.SetStartPattern(this.availablePatterns[this.startPattern.SelectedIndex]);
	}

	void autoMode_Click(object sender, EventArgs e)
	{
		if (this.doingAutoMode)
		{
			this.autoMode.Text = "Start Automode";
			this.next.Enabled = true;
			this.clear.Enabled = true;
			if (this.StartAutoMode != null && this.StopAutoMode != null)
				this.StopAutoMode();
		}
		else
		{
			int ms = 0;
			try
			{
				ms = Int32.Parse(this.msPerGeneration.Text);
				this.autoMode.Text = "Stop Automode";
				this.next.Enabled = false;
				this.clear.Enabled = false;

				if (this.StartAutoMode != null && this.StopAutoMode != null)
					this.StartAutoMode(ms);

			}
			catch(ArgumentException){
				MessageBox.Show("Der Wert fuer die Anzahl der Millisekunden / Generation ist fehlerhaft.", "Game Of Life", MessageBoxButtons.OK, MessageBoxIcon.Information);}
			catch (FormatException){
					MessageBox.Show("Der Wert fuer die Anzahl der Millisekunden / Generation ist fehlerhaft.", "Game Of Life", MessageBoxButtons.OK, MessageBoxIcon.Information);}
			catch (OverflowException){
						MessageBox.Show("Der Wert fuer die Anzahl der Millisekunden / Generation ist fehlerhaft.", "Game Of Life", MessageBoxButtons.OK, MessageBoxIcon.Information);}	
		}
		this.doingAutoMode = !this.doingAutoMode;
	}
	
	public void InvokeNrOfGeneration()
	{
		SetNrOfGenerationHandler c = new SetNrOfGenerationHandler(this.SetNuNrOfGeneration);
		this.Invoke(c, new object[] {this.gol.NrOfGeneration});
	}
	
	private void SetNuNrOfGeneration(int nr)
	{
		this.nrOfGeneration.Text = "Generation: #" + nr.ToString();
	}
}