using System;
using System.Windows.Forms;
using System.Drawing;

class GOLView : Panel
{
	// Members
	private GameOfLife gol;
	private int cellSize;
	private int margin;
	
	// C'tor
	public GOLView(GameOfLife gol)
	{
		this.gol = gol;
		this.BorderStyle = BorderStyle.FixedSingle;
		this.margin = 5;
		//this.SetStyle(ControlStyles.DoubleBuffer | ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
		//this.UpdateStyles();
		//this.SetStyle(ControlStyles.DoubleBuffer, true);
		this.DoubleBuffered = true;
	}

	protected override void OnPaint(PaintEventArgs e)
	{
		base.OnPaint(e);
		
		int nrOfCellsX = this.gol.Size.Width;
		int nrOfCellsY = this.gol.Size.Height;
		int nrOfCellsMax = (nrOfCellsX > nrOfCellsY) ? nrOfCellsX : nrOfCellsY;
		int controlLength = (this.Width > this.Height) ? this.Height : this.Width;
		this.cellSize = (controlLength - nrOfCellsMax) / nrOfCellsMax;
		
		Brush bAlive = new SolidBrush(Color.CornflowerBlue);
		Brush bDead = new SolidBrush(Color.LightGray);		
		
		for (int iy=0; iy<nrOfCellsY; iy++)
			for (int ix=0; ix<nrOfCellsX; ix++)
			{
				int x = ix * this.cellSize + ix + this.margin;
				int y = iy * this.cellSize + iy + this.margin;
				
				if (this.gol.Board[iy, ix].IsAlive)
					e.Graphics.FillRectangle(bAlive, x, y, this.cellSize, this.cellSize);
				else
					e.Graphics.FillRectangle(bDead, x, y, this.cellSize, this.cellSize);
			}
			
		bAlive.Dispose();
		bDead.Dispose();
	}

	protected override void OnMouseDown(MouseEventArgs e)
	{
		base.OnMouseDown(e);
		
		int x = (e.X - this.margin) / (this.cellSize + 1);
		int y = (e.Y - this.margin) / (this.cellSize + 1);;
		
		if (x > -1 &&
			x < this.gol.Size.Width &&
			y > -1 &&
			y < this.gol.Size.Height)
			this.gol.TouchCell(new Point(x, y));
	}
}