using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.Drawing;

static class GOLXml
{
	static bool validatorFlag = false;
	
	public static bool ValidateXml(string filename)
	{
		validatorFlag = true;
		
		//  AnlagenSteuerung.Properties.Resources.grid
		byte[] bytes = System.Text.Encoding.ASCII.GetBytes(GameOfLifeNS.Properties.Resources.Pattern8);
		MemoryStream stream = new MemoryStream(bytes);
		XmlSchema schema = XmlSchema.Read(stream, null);

		XmlTextReader tr = new XmlTextReader(filename);
		XmlValidatingReader vr = new XmlValidatingReader(tr);
		vr.Schemas.Add(schema);

		vr.ValidationType = ValidationType.Schema;

		vr.ValidationEventHandler += new ValidationEventHandler(ValidationCallBack);

		Console.WriteLine("Starting Validation");
		try
		{
			while (vr.Read()) ;
		}
		catch (XmlException e)
		{
			String s = String.Format("Error at line: {0}: {1}", e.LineNumber, e.Message);
			//MessageBox.Show(s, "Fehler beim Einlesen der XML-Datei", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
			validatorFlag = false;
		}
		finally
		{
			Console.WriteLine("End of Validation");
			vr.Close();
		}
		return validatorFlag;
	}

	private static void ValidationCallBack(object sender, ValidationEventArgs args)
	{
		String s = String.Format("\r\n\tValidation error: " + args.Message);
		//MessageBox.Show(s, "Fehler beim Einlesen der XML-Datei", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
		validatorFlag = false;
	}
	
	public static XmlDocument OpenXmlDoc(string filename)
	{
		XmlDocument xmlDoc = new XmlDocument();
		xmlDoc.Load(filename);
		return xmlDoc;
	}
	
	public static Size GetBoardSize(XmlDocument xmlDoc)
	{
		// XmlElement root = xmlDoc.DocumentElement;	 .SelectNodes("gameoflife/board")
		XmlNode height = xmlDoc.GetElementsByTagName("board")[0].Attributes.GetNamedItem("height");
		XmlNode width = xmlDoc.GetElementsByTagName("board")[0].Attributes.GetNamedItem("width");
		return new Size(Int32.Parse(height.Value), Int32.Parse(width.Value));
	}
	
	public static Point[] GetLivingCells(XmlDocument xmlDoc)
	{
		XmlNodeList cells = xmlDoc.GetElementsByTagName("cell");
		Point[] p = new Point[cells.Count];
		
		for (int i=0; i<cells.Count; i++)
		{
			int x = Int32.Parse(cells[i].Attributes.GetNamedItem("x").Value);
			int y = Int32.Parse(cells[i].Attributes.GetNamedItem("y").Value);
			
			p[i] = new Point(x, y);
		}
		
		return p;
	}
	
	public static void CloseXmlDoc(XmlDocument xmlDoc)
	{
		
	}
	
	public static void SaveBoardState(Size s, ArrayList a, string filename)
	{
		XmlTextWriter xmlTw = new XmlTextWriter(filename, null);
		xmlTw.WriteStartDocument();
		
		xmlTw.WriteRaw("\r\n");
		xmlTw.WriteStartElement("gameoflife");
		
		xmlTw.WriteRaw("\r\n\t");
		xmlTw.WriteStartElement("board");
		xmlTw.WriteAttributeString("height", s.Height.ToString());
		xmlTw.WriteAttributeString("width", s.Width.ToString());
		xmlTw.WriteEndElement();	// Board

		xmlTw.WriteRaw("\r\n\t");
		xmlTw.WriteStartElement("pattern");
		
		for (int i=0; i<a.Count; i++)
		{
			xmlTw.WriteRaw("\r\n\t\t");
			xmlTw.WriteStartElement("cell", "");
			xmlTw.WriteAttributeString("y", ((Point)a[i]).Y.ToString());
			xmlTw.WriteAttributeString("x", ((Point)a[i]).X.ToString());
			xmlTw.WriteEndElement();
		}

		xmlTw.WriteRaw("\r\n\t");
		xmlTw.WriteEndElement();	// Pattern
		
		xmlTw.WriteRaw("\r\n");
		xmlTw.WriteEndElement();	// GameOfLife
		
		xmlTw.WriteRaw("\r\n");
		xmlTw.WriteEndDocument();
		
		xmlTw.Close();
	}
}